/* @pjs preload="dogL.png", "dogR.png", "rat.png", "background.jpg", "startscreen.jpg", "gameover.jpg"; */

// import the Minim sound library
import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;

// minim controller
Minim minim;

// sound loop
AudioPlayer bgm;
AudioPlayer over;
AudioPlayer hit;
AudioPlayer bonus;

// game state
// 0 = start screen
// 1 = easy mode runnibg
// 2 = hard mode running
// 3 = game over
int state = 0;

// start screen
PImage startscreen;
PImage gameover;
PImage background;
PImage dogL;
PImage dogR;
PImage rat;
PImage cheese;

// keep track of the catcher - rat
float xPos;
float yPos;

// keep track of the catched - cheese
float cheeseX;
float cheeseY;
//float cheeseXdir = 1.0;
//float cheeseYdir = 2.0;

// keep track of the obstacle - dog
float dogX = width/2;
float dogY = height/2;
float dogXdir = 1.0;
float dogYdir = 2.0;

// keep track of scores
int points = 0;
int lives = 5;


void setup()
{
  size(500, 500);  
  
  // set up sounds
  minim = new Minim(this);
  bonus = minim.loadFile("bonus.mp3");
  hit = minim.loadFile("hit.mp3");
  over = minim.loadFile("over.mp3");
  bgm = minim.loadFile("bgm.mp3");
  bgm.loop();

  // set up images
  startscreen = loadImage("startscreen.jpg");
  background = loadImage("background.jpg");
  gameover = loadImage("gameover.jpg");
  dogL = loadImage("dogL.png");
  dogR = loadImage("dogR.png");
  rat = loadImage("rat.png");
  cheese = loadImage("cheese.png");

  playerstartpoint();
}


void draw()
{ 
  // game state #0 - start screen
  if (state == 0)
  {
    // load background screen
    image(startscreen, 0, 0, width, height);
  
    // if the user press "E"
    // move into easy mode
    if (keyPressed == true && key == 'e')
    {state = 1;}
    
    // if the user press "H"
    // move into hard mode
    if (keyPressed == true && key == 'h')
    {state = 2;}
  }
     
  else if (state == 1)
  {    
    setting();
    ratmove();
    wraparound();
    obstacle();
    getcheese();
    caughtbydog();  
  
    if (lives == 0)
    {
      state = 3;
      bgm.pause();
      over.pause();
      over.rewind();
      over.play();
    }  
  }

   else if (state == 2)
   {
    setting();
    hardmode();
    ratmove();
    wraparound();
    obstacle();
    getcheese();
    caughtbydog();  
    
    if (lives == 0)
    {
      state = 3;
      bgm.pause();
      over.pause();
      over.rewind();
      over.play();
     }  
   }
   
   // game state #3 - game over!
   else 
   {
     state = 3;
     lives = 5;
     image(gameover, 0, 0, width, height);
     textSize(20);
     fill(255, 0, 0);
     text("CLICK THE SCREEN TO PLAY AGAIN!", 70, 470);
     
     if (mousePressed)
     {
       state = 0;
       bgm.rewind();
       bgm.loop();
     }
   }
}


// functions  

void setting()
{
  // reset background
  image(background, 0, 0, width, height); 
    
  // load images
  image(cheese, cheeseX, cheeseY, 50, 50);
  image(rat, xPos, yPos, 60, 60);

  // points and lives
  fill(0);
  float time = millis();
  text("Points: " + points, 10, 450);
  text("Lives: " + lives, 10, 470);
  text("Time played: " + time, 10, 490);
}

void playerstartpoint()
{
 // for each round
 // set the rat starting point randomly from middle of 4 sides
 float chance = random(0, 100);
    
 if (chance < 25)
   {xPos = 220; yPos = 0;}
 else if (chance < 50)
   {xPos = 420; yPos = 220;}
 else if (chance < 75)
   {xPos = 220; yPos = 420;}
 else
   {xPos = 0; yPos = 220;}  
}

void obstacle()
{  
  // update dog position
  dogX += dogXdir;
  dogY += dogYdir;
  
  // constrain to canvas
  dogX = constrain(dogX, 0, width);
  dogY = constrain(dogY, 0, height);
  
  // bounce
  if (dogX >= width-80 || dogX <= 0)
  {dogXdir *= -1;}
  if (dogY >= height-80 || dogY <= 0)
  {dogYdir *= -1;}
  
  // image the correct dog
  if (dogXdir < 0)
  {image(dogL, dogX, dogY, 80, 80);}
  else
  {image(dogR, dogX, dogY, 80, 80);}  
  
}

void ratmove()
{  
  if (keyPressed && key == CODED) 
  { 
    if (keyCode == LEFT)
    {xPos -= 5;}
    else if (keyCode == RIGHT)
    {xPos += 5;}
    else if (keyCode == UP)
    {yPos -= 5;}
    else if (keyCode == DOWN)
    {yPos += 5;}
  }
}

void wraparound()
{
    if (xPos > width)
    {xPos = xPos - width;}
    if (xPos < 0)
    {xPos = width + xPos;}
    if (yPos > 0)
    {yPos = yPos - height;}
    if (yPos < 0)
    {yPos = height + yPos;}
}
    
void getcheese()
{
  // test collision between cheese and rat
  float distance = dist(xPos, yPos, cheeseX, cheeseY);
  if (distance < 50)
  {
    points ++;
    bonus.pause();
    bonus.rewind();
    bonus.play();
    
    cheeseX = random(50, width-50);
    cheeseY = random(50, height-50);
    playerstartpoint();    
   }
}
  
void caughtbydog()
{
  // test collision between rat and dog
  float distance2 = dist(xPos, yPos, dogX, dogY);
  if (distance2 < 50)
  {
    lives --;
    
    hit.pause();
    hit.rewind();
    hit.play();
    
    dogX = random(80, width-80);
    dogY = random(80, height-80);
    playerstartpoint();
  }
}

void hardmode()
{  
  // constrain the cheese movement to canvas
  cheeseX = constrain(cheeseX, 0, width);
  cheeseY = constrain(cheeseY, 0, height);    
    
  // make the cheese always run opposite to the rat
  if (xPos < cheeseX)
  {cheeseX += 1; }
  if (xPos > cheeseX)
  {cheeseX -= 1; }
  if (yPos < cheeseY)
  {cheeseY += 1;}
  if (yPos > cheeseY)
  {cheeseY -= 1; }
  
  // when cheese escapes, reset cheese position randomly
  if (cheeseX >= width-50 || cheeseX <= 0)
  {
   cheeseX = width/2;
   cheeseY = height/2;
   playerstartpoint();
   lives --;
   }
  if (cheeseY >= height-50 || cheeseY <= 0)
  {
   cheeseX = width/2;
   cheeseY = height/2;
   playerstartpoint();
   lives --;
   }  
}


